@@ -8,16 +8,16 @@ App::CLI::Extension - for App::CLI extension module
 
 =head1 VERSION
 
-0.6
+0.7
 
 =head1 SYNOPSIS
 
   # MyApp.pm
   package MyApp;
-
+  
   use strict;
   use base qw(App::CLI::Extension);
-
+  
   # extension method
   # load App::CLI::Plugin::Foo,  MyApp::Plugin::Bar
   __PACKAGE__->load_plugins(qw(Foo +MyApp::Plugin::Bar));
@@ -26,30 +26,30 @@ App::CLI::Extension - for App::CLI extension module
   __PACKAGE__->config( name => "kurt");
   
   1;
-
+  
   # MyApp/Hello.pm
   package MyApp::Hello;
-
+  
   use strict;
   use base qw(App::CLI::Command);
   use constant options => ("age=i" => "age");
-
+  
   sub run {
-
+  
       my($self, @args) = @_;
       # config - App::CLI::Extension extension method(App::CLI::Extension::Component::Config)
       print "Hello! my name is " . $self->config->{name} . "\n";
       print "age is " . "$self->{age}\n";
   }
-
+  
   # myapp
   #!/usr/bin/perl
-
+  
   use strict;
   use MyApp;
-
+  
   MyApp->dispatch;
-
+  
   # execute
   [kurt@localhost ~] myapp hello --age=27
   Hello! my name is kurt
@@ -70,16 +70,18 @@ use base qw(App::CLI Class::Data::Accessor);
 use 5.008;
 use UNIVERSAL::require;
 
-our $VERSION    = '0.6';
+our $VERSION    = '0.7';
 our @COMPONENTS = qw(
                      FirstSetup
                      Config
+					 OriginalArgv
                      Stash
                   );
 
-__PACKAGE__->mk_classaccessor( "_config"      => {} );
-__PACKAGE__->mk_classaccessor( "_components" );
-__PACKAGE__->mk_classaccessor( "_plugins" );
+__PACKAGE__->mk_classaccessor("_config"      => {});
+__PACKAGE__->mk_classaccessor("_components");
+__PACKAGE__->mk_classaccessor("_orig_argv");
+__PACKAGE__->mk_classaccessor("_plugins");
 
 =pod
 
@@ -89,33 +91,65 @@ __PACKAGE__->mk_classaccessor( "_plugins" );
 
 sub import {
 
-    my $class = shift;
-    my @loaded_components;
-    foreach my $component (@COMPONENTS) {
-       $component = sprintf "%s::Component::%s", __PACKAGE__, $component;
-       $component->require or die "load component error: $UNIVERSAL::require::ERROR";
-       $component->import;
-       push @loaded_components, $component;
+	my $class = shift;
+	my @loaded_components;
+	foreach my $component (@COMPONENTS) {
+		$component = sprintf "%s::Component::%s", __PACKAGE__, $component;
+		$component->require or die "load component error: $UNIVERSAL::require::ERROR";
+		$component->import;
+		push @loaded_components, $component;
     }
-    $class->_components(\@loaded_components);
+	$class->_components(\@loaded_components);
 }
 
 sub prepare {
 
     my $class = shift;
-    my $cmd = $class->SUPER::prepare(@_);
-    {
-        no strict "refs"; ## no critic
-        my $pkg = ref($cmd);
 
-        # component and plugin setup
-        push @{"$pkg\::ISA"}, @{$class->_components}, @{$class->_plugins};
-        $cmd->config($class->_config);
-        $cmd->setup;
-    }
-    return $cmd;
+	# save original argv
+	my @argv = @ARGV;
+	$class->_orig_argv(\@argv);
+
+	my $cmd = $class->SUPER::prepare(@_);
+	{
+		no strict "refs"; ## no critic
+		my $pkg = ref($cmd);
+		# component and plugin setup
+		push @{"$pkg\::ISA"}, @{$class->_components}, @{$class->_plugins};
+		$cmd->config($class->_config);
+		$cmd->orig_argv($class->_orig_argv);
+		$cmd->setup;
+	}
+	return $cmd;
 }
 
+## I really does not want....
+sub get_cmd {
+
+    my ($class, $cmd, @arg) = @_;
+	if (!defined $cmd || $cmd !~ /^[a-z0-9_]+$/) {
+		die $class->error_cmd;
+	}
+	#die $class->error_cmd unless $cmd && $cmd =~ m/^[?a-z]+$/;
+	
+    my $pkg = join('::', $class->command_class, $class->_cmd_map($cmd));
+	my $file = "$pkg.pm";
+	$file =~ s!::!/!g;
+	$pkg->require;
+
+	if (!$pkg->can("run")) {
+		if ($@ && exists $INC{$file}) {
+			warn $@;
+		}
+		die $class->error_cmd;
+	}
+
+    my $c = $pkg->new(@arg);
+    $c->app($class);
+    return $c;
+}
+
+
 =pod
 
 =head2 load_plugins
@@ -130,35 +164,35 @@ Example
   
   # MyApp/Plugin/GoodMorning.pm
   package MyApp::Plugin::GoodMorning;
-
-  use strict;
   
+  use strict;
+   
   sub good_morning {
-
+  
       my $self = shift;
       print "Good monring!\n";
   }
-
+  
   # MyApp/Hello.pm
   package MyApp::Hello;
-
+  
   use strict;
   use base qw(App::CLI::Command);
-
+  
   sub run {
-
+  
       my($self, @args) = @_;
       $self->good_morning;
   }
-
+  
   # myapp
   #!/usr/bin/perl
-
+  
   use strict;
   use MyApp;
-
+  
   MyApp->dispatch;
-
+  
   # execute
   [kurt@localhost ~] myapp hello
   Good morning!
@@ -167,22 +201,22 @@ Example
 
 sub load_plugins {
 
-    my($class, @load_plugins) = @_;
+	my($class, @load_plugins) = @_;
 
-    my @loaded_plugins;
-    foreach my $plugin(@load_plugins){
+	my @loaded_plugins;
+	foreach my $plugin(@load_plugins){
 
-        if($plugin =~ /^\+/){
-            $plugin =~ s/^\+//;
-        }else{
-            $plugin = "App::CLI::Plugin::$plugin";
-        }
-        $plugin->require or die "plugin load error: $UNIVERSAL::require::ERROR";
-        $plugin->import;
-        push @loaded_plugins, $plugin;
+		if ($plugin =~ /^\+/) {
+			$plugin =~ s/^\+//;
+		} else {
+			$plugin = "App::CLI::Plugin::$plugin";
+		}
+		$plugin->require or die "plugin load error: $UNIVERSAL::require::ERROR";
+		$plugin->import;
+		push @loaded_plugins, $plugin;
     }
 
-    $class->_plugins(\@loaded_plugins);
+	$class->_plugins(\@loaded_plugins);
 }
 
 =pod
@@ -202,27 +236,27 @@ Example
   
   # MyApp/Hello.pm
   package MyApp::Hello;
-
+  
   use strict;
   use base qw(App::CLI::Command);
-
+  
   sub run {
-
+  
       my($self, @args) = @_;
       print "My name is " . $self->config->{name} . "\n";
       print "My favorite group is " . $self->config->{favorite_group} . "\n";
       print "My favorite song is " . join(",", @{$self->config->{favorite_song}});
       print " and Smells Like Teen Spirit\n"
   }
-
+  
   # myapp
   #!/usr/bin/perl
-
+  
   use strict;
   use MyApp;
-
+  
   MyApp->dispatch;
-
+  
   # execute
   [kurt@localhost ~] myapp hello
   My name is kurt
@@ -233,13 +267,109 @@ Example
 
 sub config {
 
-    my($class, %config) = @_;
-    $class->_config(\%config);
-    return $class->_config;
+	my($class, %config) = @_;
+	$class->_config(\%config);
+	return $class->_config;
 }
 
 =head1 COMPONENT METHOD
 
+=head2 argv0
+
+my script name
+
+Example:
+
+  # MyApp/Hello.pm
+  package MyApp::Hello;
+  use strict;
+  use feature ":5.10.0";
+  use base qw(App::CLI::Command);
+     
+  sub run {
+  
+      my($self, @args) = @_;
+      say "my script name is " . $self->argv0;
+  }
+  
+  1;
+
+  # execute
+  [kurt@localhost ~] myapp hello
+  my script name is myapp
+
+=head2 full_argv0
+
+my script fullname
+
+Example:
+
+  # MyApp/Hello.pm
+  package MyApp::Hello;
+  use strict;
+  use feature ":5.10.0";
+  use base qw(App::CLI::Command);
+  
+  sub run {
+  
+      my($self, @args) = @_;
+      say "my script full name is " . $self->full_argv0;
+  }
+  
+  1;
+  
+  # execute
+  [kurt@localhost ~] myapp hello
+  my script name is /home/kurt/myapp
+
+=head2 cmdline
+
+my execute cmdline string
+
+Example:
+
+  # MyApp/Hello.pm
+  package MyApp::Hello;
+  use strict;
+  use feature ":5.10.0";
+  use base qw(App::CLI::Command);
+  
+  sub run {
+  
+      my($self, @args) = @_;
+      say "my script cmdline is [" . $self->cmdline . "]";
+  }
+  
+  1;
+  
+  # execute
+  [kurt@localhost ~] myapp hello --verbose --num=10
+  my script cmdline is [/home/kurt/myapp hello --verbose --num=10]
+
+=head2 orig_argv
+
+my execute script original argv
+
+Example:
+
+  # MyApp/Hello.pm
+  package MyApp::Hello;
+  use strict;
+  use feature ":5.10.0";
+  use base qw(App::CLI::Command);
+  
+  sub run {
+  
+      my($self, @args) = @_;
+      say "my script original argv is [" join(", ", @{$self->orig_argv}) . "]";
+  }
+  
+  1;
+  
+  # execute
+  [kurt@localhost ~] myapp hello --verbose --num=10
+  my script original argv is [hello,--verbose, --num=10]
+
 =head2 setup
 
 for component and plugin function
@@ -255,7 +385,7 @@ Example:
   use strict;
   use feature ":5.10.0";
   use base qw(App::CLI::Command);
-     
+   
   sub run {
   
       my($self, @args) = @_;
